sr=44100
kr=4410
ksmps=10
nchnls=2

gasig    init 0
gaflange init 0
gaphase  init 0
gachorus init 0

; Simple Sin
instr 1

  iamp init p4
  ifqc init p5
  kamp linseg 0, .002, p4, p3-.004, p4, .002, 0

  asin1 oscil kamp, ifqc, 1
  gasig =     gasig + asin1

endin

; Pluck
instr 2

  iamp =      p4*4
  ifqc =      cpspch(p5)
  kamp linseg 0, .002, p4, p3-.004, p4, .002, 0

  asin1 pluck kamp, ifqc, ifqc, p6, p7
  gasig =     gasig + asin1

endin

; Sampler Mono
instr 3

  asamp loscil  p4, p5, p6, 440, 1, p7, p8
  gasig =      gasig + asamp

endin

; Noise
instr 4

  iamp init p4
  kamp linseg 0, .002, p4, p3-.004, p4, .002, 0

  arnd1  rand  kamp
;  afilt  reson arnd1, 500, 200
  afilt  tone  arnd1, 1000
;  abalnc balance afilt, arnd1
  gasig  =     gasig + afilt

endin

; Tube Amp
instr 10

asig     init   0
kamp     linseg 0, .002, 1, p3-.004, 1, .002, 0
igaini   =      p4
igainf   =      p5
iduty    =      p6
islope   =      p7

aold     =      asig
asig     =      igaini*gasig/40000 ; Distortion effect using waveshaping.
aclip    tablei asig,5,1,.5        ; A lazy "S" curve, use table 6 for increased
aclip    =      igainf*aclip*10000 ; distortion.

; Tube Amplifiers show a shifted duty cycle which I try to emulate with
; a delay line.  The slope bit is just something goofy I thought I would add.
atemp    delayr  .1
adel1    deltapi (1-iduty*asig)/3000 + (1+islope*(asig-aold))/3000
         delayw  aclip

;         outs    adel1*kamp, adel1*kamp
  gaflange =      gaflange + adel1
  gaphase  =      gaphase  + adel1
  gachorus =      gachorus + adel1

  gasig  =       0

endin

; Flanger 1
instr 11

kamp     linseg 0, .002, 1, p3-.004, 1, .002, 0
irate    =      p4
idepth   =      p5/10000
iwave    =      p6
ifdbk    =      p7
imix     =      p8
ideloff  =      p9/10000
adel1    init   0
adel2    init   0

asig1    =      gaflange + ifdbk*adel1
asig2    =      gaflange + ifdbk*adel2

aosc1    oscil  idepth, irate, iwave
aosc1    =      aosc1+idepth+ideloff/2

aosc2    oscil  idepth, irate, iwave, .25
aosc2    =      aosc2+idepth+ideloff/2

atemp    delayr  2*idepth+ideloff
adel1    deltapi aosc1
         delayw  asig1

atemp    delayr  2*idepth+ideloff
adel2    deltapi aosc2
         delayw  asig2

         outs    (imix*adel1+asig1)*kamp, (imix*adel2+asig2)*kamp

  gaflange  =       0

endin

; Flanger 2 Cross Feedback
instr 12

kamp     linseg 0, .002, 1, p3-.004, 1, .002, 0
irate    =      p4
idepth   =      p5/10000
iwave    =      p6
ifdbk    =      p7
imix     =      p8
ideloff  =      p9/10000
adel1    init   0
adel2    init   0

asig1    =      gaflange + ifdbk*adel2
asig2    =      gaflange + ifdbk*adel1

aosc1    oscil  idepth, irate, iwave
aosc1    =      aosc1+idepth+ideloff/2

aosc2    oscil  idepth, irate, iwave, .25
aosc2    =      aosc2+idepth+ideloff/2

atemp    delayr  2*idepth+ideloff
adel1    deltapi aosc1
         delayw  asig1

atemp    delayr  2*idepth+ideloff
adel2    deltapi aosc2
         delayw  asig2

         outs    (imix*adel1+asig1)*kamp, (imix*adel2+asig2)*kamp

  gaflange  =       0

endin

; Compressor
instr 13

  ithresh = p4
  iratio  = p5

  krms  rms gasig
  kamp  =   (krms > ithresh) ? 1 : iratio
  gacmprs = kamp*gasig

endin

; Phasor
instr 14
  
kamp     linseg 0, .002, 1, p3-.004, 1, .002, 0
idel1    =      p4
imix     =      p5
ifdbk    =      p6

adel1    init   0
adel2    init   0

asig1    =      gaphase + ifdbk*adel1
asig2    =      gaphase + ifdbk*adel2

aphas1   alpass asig1, idel1, imix
aphas2   alpass asig2, idel1, imix

         outs    (imix*aphas1+imix*aphas2+asig1)*kamp, (imix*aphas1+imix*aphas2+asig2)*kamp

  gaphase  =       0

endin

; Chorus
instr 15

kamp     linseg 0, .002, 1, p3-.004, 1, .002, 0
irate    =      p4
idepth   =      p5/1000
iwave    =      p6
imix     =      p7
ideloff  =      p8/1000

adel1    init   0
adel2    init   0

asig1    =      gachorus
asig2    =      gachorus

aosc1    oscil  idepth, irate, iwave
aosc1    =      aosc1+idepth+ideloff/2

aosc2    oscil  idepth, irate, iwave, .25
aosc2    =      aosc2+idepth+ideloff/2

atemp    delayr  2*idepth+ideloff
adel1    deltapi aosc1
         delayw  asig1

atemp    delayr  2*idepth+ideloff
adel2    deltapi aosc2
         delayw  asig2

         outs    (imix*adel1+asig1)*kamp, (imix*adel2+asig2)*kamp

  gachorus  =       0

endin
