;===========================================================================
; A Multi-Effects System by Hans Mikelson
;===========================================================================

sr=44100
kr=22050
ksmps=2
nchnls=2
zakinit 30, 30

;---------------------------------------------------------------------------
; Noise
;---------------------------------------------------------------------------
instr 4

  iamp  init p4
  izout init p5
  kamp  linseg 0, .002, p4, p3-.004, p4, .002, 0

  arnd1  rand  kamp                              ;Random generator
  afilt  tone  arnd1, 1000                       ;Low pass filter
         zawm  afilt, izout                      ;Mix to zak channel

endin


;---------------------------------------------------------------------------
; Feedback Generator
;---------------------------------------------------------------------------
         instr   14

igaini   =       p4         ; Pre Gain
igainf   =       p5         ; Post Gain
iduty    =       p6         ; Duty cycle shift
itabd    =       p7         ; Distortion table
iresfqc  =       p8         ; Resonance frequency
ideltim  =       p9         ; Feedback delay time
ifeedbk  =       p10        ; Feedback gain
itabc    =       p11        ; Limiter table
izin     =       p12        ; Input channel
izout    =       p13        ; Output channel

asign    init    0          ; Initialize last value

kdclick  linseg   0, .1, 1, p3-.3, 1, .2, 0        ; Ramp feedback in and out
kamp     linseg   0, .002, 1, p3-.004, 1, .002, 0  ; Declick

asig     zar      izin                             ; Read input channel

afdbk    =        asig/100                         ; Reduce original signal
adel1    delayr   ideltim                          ; Feed back delay
afilt    butterbp adel1, iresfqc, iresfqc/4        ; Filter the delayed signal
kamprms  rms      afilt                 ; Find rms level
kampn    =        kamprms/30000         ; Normalize rms level 0-1.
kcomp    tablei   kampn,itabc,1,0        ; Look up compression value in table
         delayw   afdbk+kcomp*ifeedbk*afilt        ; Add limited feedback

         zaw     adel1, izout                   ; Write to the output channels

         endin

;---------------------------------------------------------------------------
; Mixer Section
;---------------------------------------------------------------------------
          instr 100

asig1     zar   p4             ; Read input channel 1
igl1      init  p5*p6          ; Left gain
igr1      init  p5*(1-p6)      ; Right gain

asig2     zar   p7             ; Read input channel 2
igl2      init  p8*p9          ; Left gain
igr2      init  p8*(1-p9)      ; Right gain

kdclick   linseg  0, .001, 1, p3-.002, 1, .001, 0  ; Declick

asigl     =     asig1*igl1 + asig2*igl2  ; Scale and sum
asigr     =     asig1*igr1 + asig2*igr2

          outs  kdclick*asigl, kdclick*asigr   ; Output stereo pair

          endin

;---------------------------------------------------------------------------
; Clear audio & control channels
;---------------------------------------------------------------------------
          instr 110

          zacl  0, 30          ; Clear audio channels 0 to 30
          zkcl  0, 30          ; Clear control channels 0 to 30

          endin
